@@ -24,10 +24,11 @@
 #define KEYBINDING_SEPARATOR ";"
 
 static void gail_menu_item_class_init  (GailMenuItemClass *klass);
-static void gail_menu_item_real_init   (GailWidget        *widget,
-                                        GtkWidget         *gtk_widget);
 static void gail_menu_item_object_init (GailMenuItem      *menu_item);
 
+static void                  gail_menu_item_real_initialize
+                                                          (AtkObject       *obj,
+                                                           gpointer        data);
 static gint                  gail_menu_item_get_n_children (AtkObject      *obj);
 static AtkObject*            gail_menu_item_ref_child      (AtkObject      *obj,
                                                             gint           i);
@@ -99,32 +100,28 @@ gail_menu_item_class_init (GailMenuItemClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
-  GailWidgetClass *widget_class;
-
-  widget_class = (GailWidgetClass*)klass;
 
   gobject_class->finalize = gail_menu_item_finalize;
 
-  widget_class->init = gail_menu_item_real_init;
-
   class->get_n_children = gail_menu_item_get_n_children;
   class->ref_child = gail_menu_item_ref_child;
   class->ref_state_set = gail_menu_item_ref_state_set;
+  class->initialize = gail_menu_item_real_initialize;
 
-  parent_class = g_type_class_ref (GAIL_TYPE_ITEM);
+  parent_class = g_type_class_peek_parent (klass);
 }
 
 static void
-gail_menu_item_real_init (GailWidget *widget,
-                          GtkWidget  *gtk_widget)
+gail_menu_item_real_initialize (AtkObject *obj,
+                                gpointer  data)
 {
-  GAIL_WIDGET_CLASS (parent_class)->init (widget, gtk_widget);
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  g_signal_connect (gtk_widget,
+  g_signal_connect (data,
                     "select",
                     G_CALLBACK (menu_item_select),
                     NULL);
-  g_signal_connect (gtk_widget,
+  g_signal_connect (data,
                     "deselect",
                     G_CALLBACK (menu_item_deselect),
                     NULL);
@@ -147,9 +144,8 @@ gail_menu_item_new (GtkWidget *widget)
 
   object = g_object_new (GAIL_TYPE_MENU_ITEM, NULL);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
   accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
 
   if (GTK_IS_TEAROFF_MENU_ITEM (widget))
     accessible->role = ATK_ROLE_TEAR_OFF_MENU_ITEM;
@@ -346,8 +342,14 @@ gail_menu_item_get_keybinding (AtkAction *action,
               g_return_val_if_fail (GTK_IS_TEAROFF_MENU_ITEM (temp_item), NULL);
               return NULL;
             }
-          g_return_val_if_fail (GTK_IS_ACCEL_LABEL (child), NULL);
           parent = gtk_widget_get_parent (temp_item);
+          if (!parent)
+            {
+              /*
+               * parent can be NULL when activating a window from the panel
+               */
+              return NULL;
+            }
           g_return_val_if_fail (GTK_IS_MENU_SHELL (parent), NULL);
           if (GTK_IS_MENU_BAR (parent))
             {
@@ -358,23 +360,28 @@ gail_menu_item_get_keybinding (AtkAction *action,
                 mnemonic_modifier = gtk_window_get_mnemonic_modifier (
                                        GTK_WINDOW (toplevel));
             }
-          key_val = gtk_label_get_mnemonic_keyval (GTK_LABEL (child));
-          if (key_val != GDK_VoidSymbol)
+          if (GTK_IS_LABEL (child))
             {
-              key = gtk_accelerator_name (key_val, mnemonic_modifier);
-              temp_keybinding = g_strconcat (key, full_keybinding, NULL);
-              if (temp_item == item)
-              item_keybinding = g_strdup (key); 
-              g_free (key);
-              g_free (full_keybinding);
-              full_keybinding = temp_keybinding;
-            }
-          else
-            {
-              /* No keybinding */
-              g_free (full_keybinding);
-              full_keybinding = NULL;
-              break;
+              key_val = gtk_label_get_mnemonic_keyval (GTK_LABEL (child));
+              if (key_val != GDK_VoidSymbol)
+                {
+                  key = gtk_accelerator_name (key_val, mnemonic_modifier);
+                  temp_keybinding = g_strconcat (key, full_keybinding, NULL);
+                  if (temp_item == item)
+                    {
+                      item_keybinding = g_strdup (key); 
+                    }
+                  g_free (key);
+                  g_free (full_keybinding);
+                  full_keybinding = temp_keybinding;
+                }
+              else
+                {
+                  /* No keybinding */
+                  g_free (full_keybinding);
+                  full_keybinding = NULL;
+                  break;
+                }        
             }        
           if (GTK_IS_MENU_BAR (parent))
             /* We have reached the menu bar so we are finished */
@@ -404,8 +411,11 @@ gail_menu_item_get_keybinding (AtkAction *action,
             {
               key = gtk_accel_group_find (group, find_accel, item);
 
-              accelerator = gtk_accelerator_name (key->accel_key,
-                                                  key->accel_mods);
+              if (key)
+                {           
+                  accelerator = gtk_accelerator_name (key->accel_key,
+                                                      key->accel_mods);
+                }
             }
         }
     }
