@@ -17,9 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <string.h>
 #include <gtk/gtk.h>
 #include "gailwidget.h"
 #include "gailnotebookpage.h"
+#include "gailcanvaswidget.h"
 
 static void gail_widget_class_init (GailWidgetClass *klass);
 
@@ -30,7 +32,6 @@ static void gail_widget_destroyed                (GtkWidget        *widget,
 static G_CONST_RETURN gchar* gail_widget_get_name (AtkObject *accessible);
 static G_CONST_RETURN gchar* gail_widget_get_description (AtkObject *accessible);
 static AtkObject* gail_widget_get_parent (AtkObject *accessible);
-static AtkRole gail_widget_get_role (AtkObject *accessible);
 static AtkStateSet* gail_widget_ref_state_set (AtkObject *accessible);
 static gint gail_widget_get_index_in_parent (AtkObject *accessible);
 
@@ -95,8 +96,8 @@ static gboolean   gail_widget_real_focus_gtk     (GtkWidget     *widget,
 static void       gail_widget_focus_event        (AtkObject     *obj,
                                                   gboolean      focus_in);
 
-static void       gail_widget_real_init          (GailWidget    *widget,
-                                                  GtkWidget     *gtk_widget);
+static void       gail_widget_real_initialize    (AtkObject     *obj,
+                                                  gpointer      data);
 static GtkWidget* gail_widget_find_viewport      (GtkWidget     *widget);
 static gboolean   gail_widget_on_screen          (GtkWidget     *widget);
 
@@ -108,34 +109,33 @@ gail_widget_get_type (void)
   static GType type = 0;
 
   if (!type)
-  {
-    static const GTypeInfo tinfo =
     {
-      sizeof (GailWidgetClass),
-      (GBaseInitFunc) NULL, /* base init */
-      (GBaseFinalizeFunc) NULL, /* base finalize */
-      (GClassInitFunc) gail_widget_class_init, /* class init */
-      (GClassFinalizeFunc) NULL, /* class finalize */
-      NULL, /* class data */
-      sizeof (GailWidget), /* instance size */
-      0, /* nb preallocs */
-      (GInstanceInitFunc) NULL, /* instance init */
-      NULL /* value table */
-    };
+      static const GTypeInfo tinfo =
+      {
+        sizeof (GailWidgetClass),
+        (GBaseInitFunc) NULL, /* base init */
+        (GBaseFinalizeFunc) NULL, /* base finalize */
+        (GClassInitFunc) gail_widget_class_init, /* class init */
+        (GClassFinalizeFunc) NULL, /* class finalize */
+        NULL, /* class data */
+        sizeof (GailWidget), /* instance size */
+        0, /* nb preallocs */
+        (GInstanceInitFunc) NULL, /* instance init */
+        NULL /* value table */
+      };
 
-    static const GInterfaceInfo atk_component_info =
-    {
+      static const GInterfaceInfo atk_component_info =
+      {
         (GInterfaceInitFunc) atk_component_interface_init,
         (GInterfaceFinalizeFunc) NULL,
         NULL
-    };
+      };
 
-
-    type = g_type_register_static (GTK_TYPE_ACCESSIBLE,
-                                    "GailWidget", &tinfo, 0);
-    g_type_add_interface_static (type, ATK_TYPE_COMPONENT,
-                                 &atk_component_info);
-  }
+      type = g_type_register_static (GTK_TYPE_ACCESSIBLE,
+                                     "GailWidget", &tinfo, 0);
+      g_type_add_interface_static (type, ATK_TYPE_COMPONENT,
+                                   &atk_component_info);
+    }
 
   return type;
 }
@@ -146,9 +146,8 @@ gail_widget_class_init (GailWidgetClass *klass)
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
   GtkAccessibleClass *accessible_class = GTK_ACCESSIBLE_CLASS (klass);
 
-  parent_class = g_type_class_ref (GTK_TYPE_ACCESSIBLE);
+  parent_class = g_type_class_peek_parent (klass);
 
-  klass->init = gail_widget_real_init;
   klass->notify_gtk = gail_widget_real_notify_gtk;
   klass->focus_gtk = gail_widget_real_focus_gtk;
 
@@ -157,31 +156,9 @@ gail_widget_class_init (GailWidgetClass *klass)
   class->get_name = gail_widget_get_name;
   class->get_description = gail_widget_get_description;
   class->get_parent = gail_widget_get_parent;
-  class->get_role = gail_widget_get_role;
   class->ref_state_set = gail_widget_ref_state_set;
   class->get_index_in_parent = gail_widget_get_index_in_parent;
-}
-
-/**
- * gail_widget_init:
- * @widget: a #GailWidget
- * @gtk_widget: a #GtkWidget for which the GailWidget was created.
- *
- * This function is called when implementing subclasses of #GailWidget.
- * It does initialization required for the new object. It is intended 
- * that this function is called only in the gail_..._new() methods used 
- * to create an instance of a subclass of #GailWidget
- **/
-void 
-gail_widget_init (GailWidget *widget,
-                  GtkWidget  *gtk_widget)
-{
-  GailWidgetClass *klass;
-  g_return_if_fail (GAIL_IS_WIDGET (widget));
-
-  klass = GAIL_WIDGET_GET_CLASS (widget);
-  if (klass->init)
-    klass->init (widget, gtk_widget); 
+  class->initialize = gail_widget_real_initialize;
 }
 
 /**
@@ -189,60 +166,61 @@ gail_widget_init (GailWidget *widget,
  * and specifies a handler to be called when the GtkWidget is destroyed.
  **/
 static void 
-gail_widget_real_init  (GailWidget *widget,
-                        GtkWidget  *gtk_widget)
+gail_widget_real_initialize (AtkObject *obj,
+                             gpointer  data)
 {
   GtkAccessible *accessible;
+  GtkWidget *widget;
 
-  accessible = GTK_ACCESSIBLE (widget);
-  accessible->widget = gtk_widget;
+  g_return_if_fail (GTK_IS_WIDGET (data));
+
+  widget = GTK_WIDGET (data);
+  obj->layer = ATK_LAYER_WIDGET;
+
+  accessible = GTK_ACCESSIBLE (obj);
+  accessible->widget = widget;
   gtk_accessible_connect_widget_destroyed (accessible);
-  g_signal_connect_after (GTK_OBJECT (gtk_widget),
+  g_signal_connect_after (widget,
                           "focus-in-event",
                           G_CALLBACK (gail_widget_focus_gtk),
                           NULL);
-  g_signal_connect_after (G_OBJECT (gtk_widget),
+  g_signal_connect_after (widget,
                           "focus-out-event",
                           G_CALLBACK (gail_widget_focus_gtk),
                           NULL);
-  g_signal_connect (G_OBJECT (gtk_widget),
+  g_signal_connect (widget,
                     "notify",
                     G_CALLBACK (gail_widget_notify_gtk),
                     widget);
-  /*
-   * Support state property change for focus events
-   */
   atk_component_add_focus_handler (ATK_COMPONENT (accessible),
                                    gail_widget_focus_event);
   /*
    * Add signal handlers for GTK signals required to support property changes
    */
-  g_signal_connect (G_OBJECT (gtk_widget),
+  g_signal_connect (widget,
                     "map",
                     G_CALLBACK (gail_widget_map_gtk),
                     NULL);
-  g_signal_connect (G_OBJECT (gtk_widget),
+  g_signal_connect (widget,
                     "unmap",
                     G_CALLBACK (gail_widget_map_gtk),
                     NULL);
 }
 
-GtkAccessible* 
+AtkObject* 
 gail_widget_new (GtkWidget *widget)
 {
   GObject *object;
-  GtkAccessible *accessible;
+  AtkObject *accessible;
 
   g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
   object = g_object_new (GAIL_TYPE_WIDGET, NULL);
 
-  g_return_val_if_fail (GTK_IS_ACCESSIBLE (object), NULL);
+  accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
-  accessible = GTK_ACCESSIBLE (object);
-  ATK_OBJECT(accessible)->role = ATK_ROLE_UNKNOWN;
+  accessible->role = ATK_ROLE_UNKNOWN;
 
   return accessible;
 }
@@ -255,96 +233,78 @@ static void
 gail_widget_connect_widget_destroyed (GtkAccessible *accessible)
 {
   if (accessible->widget)
-  {
-    g_signal_connect (G_OBJECT (accessible->widget),
-                      "destroy",
-                      G_CALLBACK (gail_widget_destroyed),
-                      accessible);
-  }
+    {
+      g_signal_connect (accessible->widget,
+                        "destroy",
+                        G_CALLBACK (gail_widget_destroyed),
+                        accessible);
+    }
 }
 
 /*
  * This function is called when the widget is destroyed.
  * It sets the widget field in the GtkAccessible structure to NULL
- * and signals a property change event for "accessible-state" with
- * the new state being ATK_STATE_DEFUNCT
+ * and emits a tate-change signal for the state ATK_STATE_DEFUNCT
  */
 static void 
 gail_widget_destroyed (GtkWidget     *widget,
                        GtkAccessible *accessible)
 {
-  AtkPropertyValues  values;
-
-  accessible->widget = NULL;
-
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
-  values.property_name = "accessible-state";
-  g_value_init (&values.new_value, G_TYPE_INT);
-  g_value_set_int (&values.new_value, ATK_STATE_DEFUNCT);
-  g_signal_emit_by_name (ATK_OBJECT (accessible), "property-change",
-                 &values, NULL);
+  atk_object_notify_state_change (ATK_OBJECT (accessible), ATK_STATE_DEFUNCT,
+                                  TRUE); 
 }
 
 static G_CONST_RETURN gchar*
 gail_widget_get_name (AtkObject *accessible)
 {
   if (accessible->name)
-  {
     return accessible->name;
-  }
   else
-  {
-    /*
-     * Get the widget name is it exists
-     */
-    GtkWidget* widget = GTK_ACCESSIBLE (accessible)->widget;
-
-    if (widget == NULL)
     {
       /*
-       * State is defunct
+       * Get the widget name is it exists
        */
-      return NULL;
-    }
-    g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+      GtkWidget* widget = GTK_ACCESSIBLE (accessible)->widget;
 
-    return widget->name;
-  }
+      if (widget == NULL)
+        /*
+         * State is defunct
+         */
+        return NULL;
+
+      g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+
+      return widget->name;
+    }
 }
 
 static G_CONST_RETURN gchar*
 gail_widget_get_description (AtkObject *accessible)
 {
   if (accessible->description)
-  {
     return accessible->description;
-  }
   else
-  {
-    /* Get the tooltip from the widget */
-    GtkAccessible *obj = GTK_ACCESSIBLE (accessible);
-    GtkTooltipsData *data;
-
-    g_return_val_if_fail (obj, NULL);
-
-    if (obj->widget == NULL)
-    {
-      /*
-       * Object is defunct
-       */
-      return NULL;
-    }
-    g_return_val_if_fail (GTK_WIDGET (obj->widget), NULL);
-    
-    data = gtk_tooltips_data_get(obj->widget);
-    if (data == NULL)
     {
+      /* Get the tooltip from the widget */
+      GtkAccessible *obj = GTK_ACCESSIBLE (accessible);
+      GtkTooltipsData *data;
+
+      g_return_val_if_fail (obj, NULL);
+
+      if (obj->widget == NULL)
+        /*
+         * Object is defunct
+         */
+        return NULL;
+ 
+      g_return_val_if_fail (GTK_WIDGET (obj->widget), NULL);
+    
+      data = gtk_tooltips_data_get (obj->widget);
+      if (data == NULL)
         return NULL;
-    }
 
-    return data->tip_text;
-  }
+      return data->tip_text;
+    }
 }
 
 static AtkObject* 
@@ -355,36 +315,27 @@ gail_widget_get_parent (AtkObject *accessible)
   parent = accessible->accessible_parent;
 
   if (parent != NULL)
-  {
     g_return_val_if_fail (ATK_IS_OBJECT (parent), NULL);
-  }
   else
-  {
-    GtkWidget *widget, *parent_widget;
-
-    widget = GTK_ACCESSIBLE (accessible)->widget;
-    if (widget == NULL)
     {
-      /*
-       * State is defunct
-       */
-      return NULL;
+      GtkWidget *widget, *parent_widget;
+
+      widget = GTK_ACCESSIBLE (accessible)->widget;
+      if (widget == NULL)
+        /*
+         * State is defunct
+         */
+        return NULL;
+
+      parent_widget = widget->parent;
+      if (parent_widget == NULL)
+        return NULL;
+
+      parent = gtk_widget_get_accessible (parent_widget);
     }
-    parent_widget = widget->parent;
-    if (parent_widget == NULL)
-      return NULL;
-
-    parent = gtk_widget_get_accessible (parent_widget);
-  }
   return parent;
 }
 
-static AtkRole 
-gail_widget_get_role (AtkObject *accessible)
-{
-  return accessible->role;
-}
-
 static AtkStateSet*
 gail_widget_ref_state_set (AtkObject *accessible)
 {
@@ -394,101 +345,115 @@ gail_widget_ref_state_set (AtkObject *accessible)
   state_set = ATK_OBJECT_CLASS (parent_class)->ref_state_set (accessible);
 
   if (widget == NULL)
-  {
-    atk_state_set_add_state (state_set, ATK_STATE_DEFUNCT);
-  }
+    {
+      atk_state_set_add_state (state_set, ATK_STATE_DEFUNCT);
+    }
   else
-  {
-    if (GTK_WIDGET_IS_SENSITIVE (widget))
     {
-      atk_state_set_add_state (state_set, ATK_STATE_SENSITIVE);
-    }
-  
-    if (GTK_WIDGET_CAN_FOCUS (widget))
-    {
-      atk_state_set_add_state (state_set, ATK_STATE_FOCUSABLE);
-    }
-    /*
-     * We do not currently generate notifications when an ATK object 
-     * corresponding to a GtkWidget changes visibility by being scrolled 
-     * on or off the screen.  The testcase for this is the main window 
-     * of the testgtk application in which a set of buttons in a GtkVBox 
-     * is in a scrooled window with a viewport.
-     *
-     * To generate the notifications we would need to do the following: 
-     * 1) Find the GtkViewPort among the antecendents of the objects
-     * 2) Create an accesible for the GtkViewPort
-     * 3) Connect to the value-changed signal on the viewport
-     * 4) When the signal is received we need to traverse the children 
-     * of the viewport and check whether the children are visible or not 
-     * visible; we may want to restrict this to the widgets for which 
-     * accessible objects have been created.
-     * 5) We probably need to store a variable on_screen in the 
-     * GailWidget data structure so we can determine whether the value has 
-     * changed.
-     */
-    if (gail_widget_on_screen (widget))
-    {
-      if (GTK_WIDGET_VISIBLE (widget))
-      {
-        atk_state_set_add_state (state_set, ATK_STATE_VISIBLE);
-
-        if (GTK_WIDGET_MAPPED (widget))
+      if (GTK_WIDGET_IS_SENSITIVE (widget))
         {
-          atk_state_set_add_state (state_set, ATK_STATE_SHOWING);
+          atk_state_set_add_state (state_set, ATK_STATE_SENSITIVE);
         }
-      }
-    }
+  
+      if (GTK_WIDGET_CAN_FOCUS (widget))
+        {
+          atk_state_set_add_state (state_set, ATK_STATE_FOCUSABLE);
+        }
+      /*
+       * We do not currently generate notifications when an ATK object 
+       * corresponding to a GtkWidget changes visibility by being scrolled 
+       * on or off the screen.  The testcase for this is the main window 
+       * of the testgtk application in which a set of buttons in a GtkVBox 
+       * is in a scrooled window with a viewport.
+       *
+       * To generate the notifications we would need to do the following: 
+       * 1) Find the GtkViewPort among the antecendents of the objects
+       * 2) Create an accesible for the GtkViewPort
+       * 3) Connect to the value-changed signal on the viewport
+       * 4) When the signal is received we need to traverse the children 
+       * of the viewport and check whether the children are visible or not 
+       * visible; we may want to restrict this to the widgets for which 
+       * accessible objects have been created.
+       * 5) We probably need to store a variable on_screen in the 
+       * GailWidget data structure so we can determine whether the value has 
+       * changed.
+       */
+      if (gail_widget_on_screen (widget))
+        {
+          if (GTK_WIDGET_VISIBLE (widget))
+            {
+              atk_state_set_add_state (state_set, ATK_STATE_VISIBLE);
 
-    if (GTK_WIDGET_HAS_FOCUS (widget))
-    {
-      atk_state_set_add_state (state_set, ATK_STATE_FOCUSED);
+              if (GTK_WIDGET_MAPPED (widget))
+                {
+                  atk_state_set_add_state (state_set, ATK_STATE_SHOWING);
+                }
+            }
+        }
+  
+      if (GTK_WIDGET_HAS_FOCUS (widget))
+        {
+          atk_state_set_add_state (state_set, ATK_STATE_FOCUSED);
+        }
+      atk_state_set_add_state (state_set, ATK_STATE_ENABLED);
     }
-    atk_state_set_add_state (state_set, ATK_STATE_ENABLED);
-  }
   return state_set;
 }
 
 static gint
 gail_widget_get_index_in_parent (AtkObject *accessible)
 {
-  GtkWidget *widget, *parent_widget;
-  gint index = -1;
-  GList *children, *tmp_list;
-
-  if (accessible->accessible_parent)
-  {
-    if (GAIL_IS_NOTEBOOK_PAGE (accessible->accessible_parent))
-      return 0;
-  }
+  GtkWidget *widget;
+  GtkWidget *parent_widget;
+  gint index;
+  GList *children;
 
   widget = GTK_ACCESSIBLE (accessible)->widget;
 
   if (widget == NULL)
-  {
     /*
      * State is defunct
      */
     return -1;
-  }
+
+  if (accessible->accessible_parent)
+    {
+      AtkObject *parent;
+
+      parent = accessible->accessible_parent;
+
+      if (GAIL_IS_NOTEBOOK_PAGE (parent) ||
+          GAIL_IS_CANVAS_WIDGET (parent))
+        return 0;
+      else
+        {
+          gint n_children, i;
+          gboolean found = FALSE;
+
+          n_children = atk_object_get_n_accessible_children (parent);
+          for (i = 0; i < n_children; i++)
+            {
+              AtkObject *child;
+
+              child = atk_object_ref_accessible_child (parent, i);
+              if (child == accessible)
+                found = TRUE;
+
+              g_object_unref (child); 
+              if (found)
+                return i;
+            }
+        }
+    }
+
   parent_widget = widget->parent;
   g_return_val_if_fail (GTK_IS_CONTAINER (parent_widget), -1);
 
-  tmp_list = children = gtk_container_children (GTK_CONTAINER (parent_widget));
+  children = gtk_container_children (GTK_CONTAINER (parent_widget));
 
-  while (tmp_list)
-  {
-    if (GTK_WIDGET (tmp_list->data) == widget)
-    {
-      index++;
-      g_list_free (children);
-      return index;
-    }
-    tmp_list = g_list_next (tmp_list);
-    index++;
-  }
+  index = g_list_index (children, widget);
   g_list_free (children);
-  return -1;  
+  return index;  
 }
 
 static void 
@@ -513,12 +478,28 @@ static guint
 gail_widget_add_focus_handler (AtkComponent    *component,
                                AtkFocusHandler handler)
 {
-  return g_signal_connect_closure (component, 
-                                   "focus-event",
-                                   g_cclosure_new (
-                                   G_CALLBACK (handler), NULL,
-                                   (GClosureNotify) NULL),
-                                   FALSE);
+  GSignalMatchType match_type;
+  gulong ret;
+  guint signal_id;
+
+  match_type = G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC;
+  signal_id = g_signal_lookup ("focus-event", ATK_TYPE_OBJECT);
+
+  ret = g_signal_handler_find (component, match_type, signal_id, 0, NULL,
+                               handler, NULL);
+  if (!ret)
+    {
+      return g_signal_connect_closure_by_id (component, 
+                                             signal_id, 0,
+                                             g_cclosure_new (
+                                             G_CALLBACK (handler), NULL,
+                                             (GClosureNotify) NULL),
+                                             FALSE);
+    }
+  else
+    {
+      return 0;
+    }
 }
 
 static void 
@@ -530,84 +511,80 @@ gail_widget_get_extents (AtkComponent   *component,
                          AtkCoordType   coord_type)
 {
   GdkWindow *window;
-  gint x_parent, y_parent, x_toplevel, y_toplevel;
+  gint x_window, y_window;
+  gint x_toplevel, y_toplevel;
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
 
   if (widget == NULL)
-  {
     /*
      * Object is defunct
      */
     return;
-  }
 
   *width = widget->allocation.width;
   *height = widget->allocation.height;
   if (!gail_widget_on_screen (widget) || (!GTK_WIDGET_DRAWABLE (widget)))
-  {
-    *x = G_MININT;
-    *y = G_MININT;
-    return;
-  }
+    {
+      *x = G_MININT;
+      *y = G_MININT;
+      return;
+    }
 
   if (widget->parent)
-  {
-    window = gtk_widget_get_parent_window(widget);
-  }
+    {
+      *x = widget->allocation.x;
+      *y = widget->allocation.y;
+      window = gtk_widget_get_parent_window (widget);
+    }
   else
-  {
-    window = widget->window;
-  }
-  {
-    int x, y, width, height, depth;
-    gdk_window_get_position (window, &x, &y);
-    gdk_window_get_geometry (window, &x, &y, &width, &height, &depth);
-  }
-  gdk_window_get_origin(window, &x_parent, &y_parent);
-  window = gdk_window_get_toplevel(widget->window);
-  gdk_window_get_origin(window, &x_toplevel, &y_toplevel);
+    {
+      *x = 0;
+      *y = 0;
+      window = widget->window;
+    }
+  gdk_window_get_origin (window, &x_window, &y_window);
+  *x += x_window;
+  *y += y_window;
 
-  if (coord_type == ATK_XY_SCREEN) 
-  { 
-    *x = widget->allocation.x + x_parent;
-    *y = widget->allocation.y + y_parent;
-  }
-  else if (coord_type == ATK_XY_WINDOW) 
-  { 
-    *x = widget->allocation.x + (x_parent - x_toplevel);
-    *y = widget->allocation.y + (y_parent - y_toplevel);
-  }
+ 
+ if (coord_type == ATK_XY_WINDOW) 
+    { 
+      window = gdk_window_get_toplevel (widget->window);
+      gdk_window_get_origin (window, &x_toplevel, &y_toplevel);
+
+      *x -= x_toplevel;
+      *y -= y_toplevel;
+    }
 }
 
 static void 
-gail_widget_get_size    (AtkComponent   *component,
-                         gint           *width,
-                         gint           *height)
+gail_widget_get_size (AtkComponent   *component,
+                      gint           *width,
+                      gint           *height)
 {
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
 
   if (widget == NULL)
-  {
     /*
      * Object is defunct
      */
     return;
-  }
+
   *width = widget->allocation.width;
   *height = widget->allocation.height;
 }
 
 static gboolean 
-gail_widget_grab_focus(AtkComponent   *component)
+gail_widget_grab_focus (AtkComponent   *component)
 {
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
 
   g_return_val_if_fail (widget != NULL, FALSE);
-  if (GTK_WIDGET_CAN_FOCUS(widget))
-  {
-    gtk_widget_grab_focus (widget);
-    return TRUE;
-  }
+  if (GTK_WIDGET_CAN_FOCUS (widget))
+    {
+      gtk_widget_grab_focus (widget);
+      return TRUE;
+    }
   else
     return FALSE;
 }
@@ -616,7 +593,7 @@ static void
 gail_widget_remove_focus_handler (AtkComponent   *component,
                                   guint          handler_id)
 {
-  g_signal_handler_disconnect (ATK_OBJECT (component), handler_id);
+  g_signal_handler_disconnect (component, handler_id);
 }
 
 static gboolean 
@@ -630,100 +607,99 @@ gail_widget_set_extents (AtkComponent   *component,
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
 
   if (widget == NULL)
-  {
-    /*gtkwindow.c
+    /*
      * Object is defunct
      */
     return FALSE;
-  }
-  if (GTK_WIDGET_TOPLEVEL(widget))
-  {
-    if (coord_type == ATK_XY_WINDOW)
+
+  if (GTK_WIDGET_TOPLEVEL (widget))
     {
-      gint x_current, y_current;
-      GdkWindow *window = widget->window;
-      gdk_window_get_origin(window, &x_current, &y_current);
-      x_current += x;
-      y_current += y;
-      if (x_current < 0 || y_current < 0)
-        return FALSE;
-      else
-      {
-        gtk_widget_set_uposition (widget, x_current, y_current);
-        gtk_widget_set_usize (widget, width, height);
-        return TRUE;
-      }
+      if (coord_type == ATK_XY_WINDOW)
+        {
+          gint x_current, y_current;
+          GdkWindow *window = widget->window;
+
+          gdk_window_get_origin (window, &x_current, &y_current);
+          x_current += x;
+          y_current += y;
+          if (x_current < 0 || y_current < 0)
+            return FALSE;
+          else
+            {
+              gtk_widget_set_uposition (widget, x_current, y_current);
+              gtk_widget_set_usize (widget, width, height);
+              return TRUE;
+            }
+        }
+      else if (coord_type == ATK_XY_SCREEN)
+        {  
+          gtk_widget_set_uposition (widget, x, y);
+          gtk_widget_set_usize (widget, width, height);
+          return TRUE;
+        }
     }
-    else if (coord_type == ATK_XY_SCREEN)
-    {  
-      gtk_widget_set_uposition (widget, x, y);
-      gtk_widget_set_usize (widget, width, height);
-      return TRUE;
-    }
-  }
   return FALSE;
 }
 
 static gboolean
-gail_widget_set_position(AtkComponent   *component,
-                         gint           x,
-                         gint           y,
-                         AtkCoordType   coord_type)
+gail_widget_set_position (AtkComponent   *component,
+                          gint           x,
+                          gint           y,
+                          AtkCoordType   coord_type)
 {
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
 
   if (widget == NULL)
-  {
     /*
      * Object is defunct
      */
     return FALSE;
-  }
-  if (GTK_WIDGET_TOPLEVEL(widget))
-  {
-    if (coord_type == ATK_XY_WINDOW)
+
+  if (GTK_WIDGET_TOPLEVEL (widget))
     {
-      gint x_current, y_current;
-      GdkWindow *window = widget->window;
-      gdk_window_get_origin(window, &x_current, &y_current);
-      x_current += x;
-      y_current += y;
-      if (x_current < 0 || y_current < 0)
-        return FALSE;
-      else
-      {
-        gtk_widget_set_uposition (widget, x_current, y_current);
-        return TRUE;
-      }
+      if (coord_type == ATK_XY_WINDOW)
+        {
+          gint x_current, y_current;
+          GdkWindow *window = widget->window;
+
+          gdk_window_get_origin (window, &x_current, &y_current);
+          x_current += x;
+          y_current += y;
+          if (x_current < 0 || y_current < 0)
+            return FALSE;
+          else
+            {
+              gtk_widget_set_uposition (widget, x_current, y_current);
+              return TRUE;
+            }
+        }
+      else if (coord_type == ATK_XY_SCREEN)
+        {  
+          gtk_widget_set_uposition (widget, x, y);
+          return TRUE;
+        }
     }
-    else if (coord_type == ATK_XY_SCREEN)
-    {  
-      gtk_widget_set_uposition (widget, x, y);
-      return TRUE;
-    }
-  }
   return FALSE;
 }
 
 static gboolean 
-gail_widget_set_size    (AtkComponent   *component,
-                         gint           width,
-                         gint           height)
+gail_widget_set_size (AtkComponent   *component,
+                      gint           width,
+                      gint           height)
 {
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
 
   if (widget == NULL)
-  {
     /*
      * Object is defunct
      */
     return FALSE;
-  }
-  if (GTK_WIDGET_TOPLEVEL(widget))
-  {
-    gtk_widget_set_usize (widget, width, height);
-    return TRUE;
-  }
+
+  if (GTK_WIDGET_TOPLEVEL (widget))
+    {
+      gtk_widget_set_usize (widget, width, height);
+      return TRUE;
+    }
   else
    return FALSE;
 }
@@ -734,15 +710,13 @@ _gail_widget_get_parent (GtkWidget* widget)
 {
   GtkWidget *parent_widget;
   gpointer pointer;
-  GValue *value, val;
+  GValue value = { 0, };
 
   g_return_val_if_fail (GTK_WIDGET (widget), NULL);
 
-  value = &val;
-  memset (value, 0, sizeof (value));
-  g_value_init (value, G_TYPE_OBJECT);
-  g_object_get_property (G_OBJECT (widget), "parent", value);
-  pointer = g_value_get_object (value);
+  g_value_init (&value, G_TYPE_OBJECT);
+  g_object_get_property (G_OBJECT (widget), "parent", &value);
+  pointer = g_value_get_object (&value);
  
   g_return_val_if_fail (GTK_IS_WIDGET (pointer), NULL);
 
@@ -750,7 +724,6 @@ _gail_widget_get_parent (GtkWidget* widget)
 }
 #endif
 
-
 /*
  * This function is a signal handler for notify_in_event and focus_out_event
  * signal which gets emitted on a GtkWidget.
@@ -785,8 +758,7 @@ gail_widget_real_focus_gtk (GtkWidget     *widget,
   return_val = FALSE;
 
   accessible = gtk_widget_get_accessible (widget);
-  g_signal_emit_by_name (G_OBJECT (accessible), "focus-event", 
-                 event->in, &return_val);
+  g_signal_emit_by_name (accessible, "focus_event", event->in, &return_val);
   return FALSE;
 }
 
@@ -797,25 +769,10 @@ static gint
 gail_widget_map_gtk (GtkWidget     *widget)
 {
   AtkObject* accessible;
-  AtkPropertyValues  values;
 
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
-  values.property_name = "accessible-state";
-  if (GTK_WIDGET_MAPPED (widget))
-  {
-     g_value_init (&values.new_value, G_TYPE_INT);
-     g_value_set_int (&values.new_value, ATK_STATE_SHOWING);
-  }
-  else
-  { 
-     g_value_init (&values.old_value, G_TYPE_INT);
-     g_value_set_int (&values.old_value, ATK_STATE_SHOWING);
-  }
   accessible = gtk_widget_get_accessible (widget);
-  g_signal_emit_by_name (accessible, "property-change",
-                 &values, NULL);
-
+  atk_object_notify_state_change (accessible, ATK_STATE_SHOWING,
+                                  GTK_WIDGET_MAPPED (widget));
   return 1;
 }
 
@@ -850,110 +807,38 @@ static void
 gail_widget_real_notify_gtk (GObject     *obj,
                              GParamSpec  *pspec)
 {
-  AtkPropertyValues values;
   GtkWidget* widget = GTK_WIDGET (obj);
   AtkObject* atk_obj = gtk_widget_get_accessible (widget);
+  AtkState state;
+  gboolean value;
 
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
   if (strcmp (pspec->name, "visible") == 0)
-  {
-    if (GTK_WIDGET_VISIBLE (widget))
     {
-      g_value_init (&values.new_value, G_TYPE_INT);
-      g_value_set_int (&values.new_value, ATK_STATE_VISIBLE);
-    } 
-    else
-    {
-      g_value_init (&values.old_value, G_TYPE_INT);
-      g_value_set_int (&values.old_value, ATK_STATE_VISIBLE);
-    } 
-  }
+      state = ATK_STATE_VISIBLE;
+      value = GTK_WIDGET_VISIBLE (widget);
+    }
   else if (strcmp (pspec->name, "has-focus") == 0)
-  {
     /*
      * We use focus-in-event and focus-out-event signals to catch
      * focus changes so we ignore this.
      */
     return;
-  }
   else if (strcmp (pspec->name, "sensitive") == 0)
-  {
-    if (GTK_WIDGET_SENSITIVE (widget))
     {
-      g_value_init (&values.new_value, G_TYPE_INT);
-      g_value_set_int (&values.new_value, ATK_STATE_SENSITIVE);
-    } 
-    else
-    {
-      g_value_init (&values.old_value, G_TYPE_INT);
-      g_value_set_int (&values.old_value, ATK_STATE_SENSITIVE);
-    } 
-  }
+      state = ATK_STATE_SENSITIVE;
+      value = GTK_WIDGET_SENSITIVE (widget);
+    }
   else
     return;
 
-  values.property_name = "accessible-state";
-  g_signal_emit_by_name (G_OBJECT (atk_obj), "property-change",
-                 &values, NULL);
+  atk_object_notify_state_change (atk_obj, state, value);
 }
 
-/*
- * This function is the signal handler defined for focus event on
- * AtkObject by the property change mechanism so that we get a
- * property change for accessible-state when focus changes.
- */
 static void 
 gail_widget_focus_event (AtkObject   *obj,
                          gboolean    focus_in)
 {
-  AtkPropertyValues  values;
-
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
-  values.property_name = "accessible-state";
-  if (focus_in)
-  {
-     g_value_init (&values.new_value, G_TYPE_INT);
-     g_value_set_int (&values.new_value, ATK_STATE_FOCUSED);
-  }
-  else
-  { 
-     g_value_init (&values.old_value, G_TYPE_INT);
-     g_value_set_int (&values.old_value, ATK_STATE_FOCUSED);
-  }
-  g_signal_emit_by_name (obj, "property-change", &values, NULL);
-}
-
-/*
- * The following function is general purpose, so it might make sense to move
- * it to a general purpose utility file at some point.
- */
-
-/**
- * get_widget_and_window_origin:
- * @widget: a #GtkWidget
- * @x_widget: the x-origin of the widget->window
- * @y_widget: the y-origin of the widget->window
- * @x_window: the top-level x-origin of the widget->window
- * @y_window: the top-level y-origin of the widget->window
- *
- * Gets the origin of the widget window, and the origin of the
- * widgets top-level window.
- **/
-void
-gail_widget_get_origins(GtkWidget	*widget,
-                             gint	*x_widget,
-                             gint	*y_widget,
-                             gint	*x_window,
-                             gint	*y_window)
-{
-  GdkWindow *window;
-
-  gdk_window_get_origin(GDK_WINDOW(widget->window),
-    x_widget, y_widget);
-  window = gdk_window_get_toplevel(GDK_WINDOW(widget->window));
-  gdk_window_get_origin(window, x_window, y_window);
+  atk_object_notify_state_change (obj, ATK_STATE_FOCUSED, focus_in);
 }
 
 static GtkWidget*
@@ -966,11 +851,11 @@ gail_widget_find_viewport (GtkWidget *widget)
 
   parent = widget->parent;
   while (parent != NULL)
-  {
-    if (GTK_IS_VIEWPORT (parent))
-      break;
-    parent = parent->parent;
-  }
+    {
+      if (GTK_IS_VIEWPORT (parent))
+        break;
+      parent = parent->parent;
+    }
   return parent;
 }
 
@@ -986,32 +871,27 @@ static gboolean gail_widget_on_screen (GtkWidget *widget)
 
   viewport = gail_widget_find_viewport (widget);
   if (viewport)
-  {
-    GtkAdjustment *adjustment;
-    GdkRectangle visible_rect;
+    {
+      GtkAdjustment *adjustment;
+      GdkRectangle visible_rect;
 
-    adjustment = gtk_viewport_get_vadjustment (GTK_VIEWPORT (viewport));
-    visible_rect.y = adjustment->value;
-    adjustment = gtk_viewport_get_hadjustment (GTK_VIEWPORT (viewport));
-    visible_rect.x = adjustment->value;
-    visible_rect.width = viewport->allocation.width;
-    visible_rect.height = viewport->allocation.height;
+      adjustment = gtk_viewport_get_vadjustment (GTK_VIEWPORT (viewport));
+      visible_rect.y = adjustment->value;
+      adjustment = gtk_viewport_get_hadjustment (GTK_VIEWPORT (viewport));
+      visible_rect.x = adjustment->value;
+      visible_rect.width = viewport->allocation.width;
+      visible_rect.height = viewport->allocation.height;
              
-    if (((widget->allocation.x + widget->allocation.width) < visible_rect.x) ||
-       ((widget->allocation.y + widget->allocation.height) < visible_rect.y) ||
-       (widget->allocation.x > (visible_rect.x + visible_rect.width)) ||
-       (widget->allocation.y > (visible_rect.y + visible_rect.height)))
-    {
-      return_value = FALSE;
+      if (((widget->allocation.x + widget->allocation.width) < visible_rect.x) ||
+         ((widget->allocation.y + widget->allocation.height) < visible_rect.y) ||
+         (widget->allocation.x > (visible_rect.x + visible_rect.width)) ||
+         (widget->allocation.y > (visible_rect.y + visible_rect.height)))
+        return_value = FALSE;
+      else
+        return_value = TRUE;
     }
-    else
-    {
-      return_value = TRUE;
-    }
-  }
   else
-  {
     return_value = TRUE;
-  }
+
   return return_value;
 }
